require 'spec_helper'

describe Hash do
  let(:hash_empty) { {} }
  let(:hash_not_empty) { { hoge: 1, hoge2: 2} }
  let(:deep_hash) { { first: { second: { third: 1 }} } }

  def expectation_value(joint_box = ".")
    { "first#{joint_box}second#{joint_box}third" => 1 }
  end

  [:deep_merge, :deep_merge!, :join_keys, :join_keys!].each do |method|
    it "has #{method} method" do
      Hash.new.should respond_to(method)
    end
  end

  it "return a new hash with self and other merged recursively" do
    hash_empty_dup = hash_empty.dup
    hash_empty.deep_merge hash_not_empty

    hash_empty.should eql(hash_empty_dup)
  end

  it "return a hash with self and other merged recursively" do
    hash_empty_dup = hash_empty.dup
    hash_empty.deep_merge! hash_not_empty

    hash_empty.should_not eql(hash_empty_dup)
    hash_empty.should eql(hash_not_empty)
  end

  it "return a string with join value into hash" do
    deep_hash_clone = deep_hash.clone
    deep_hash.join_keys(".").should eql expectation_value
    deep_hash.dup.should eql deep_hash_clone.dup

    deep_hash.join_keys!(".").should eql expectation_value
    deep_hash.should_not eql deep_hash_clone
  end

  context :join_keys do
    keys = %w[# . -> & = _]
    keys.each do |key|
      it "return hash key joined with #{key}" do
        deep_hash.join_keys(key).should eql expectation_value(key)
      end
    end
  end

end
